Exception: MPV::ReleasedConnectionError

Inherits:
ConnectionError show all
Defined in:
lib/mpv_ipc/exceptions.rb

Overview

Raised when the connection to mpv has already been released.

Instance Method Summary collapse

Constructor Details

#initializeReleasedConnectionError

Returns a new instance of ReleasedConnectionError.



30
31
32
# File 'lib/mpv_ipc/exceptions.rb', line 30

def initialize
  super "Connection released, further communication with mpv is not possible"
end