Exception: MPV::ReleasedConnectionError
- Inherits:
-
ConnectionError
- Object
- RuntimeError
- Error
- ConnectionError
- MPV::ReleasedConnectionError
- Defined in:
- lib/mpv_ipc/exceptions.rb
Overview
Raised when the connection to mpv has already been released.
Instance Method Summary collapse
-
#initialize ⇒ ReleasedConnectionError
constructor
A new instance of ReleasedConnectionError.
Constructor Details
#initialize ⇒ ReleasedConnectionError
Returns a new instance of ReleasedConnectionError.
30 31 32 |
# File 'lib/mpv_ipc/exceptions.rb', line 30 def initialize super "Connection released, further communication with mpv is not possible" end |